package com.hand.hap.core.web.view.ui;

import com.hand.hap.core.web.view.ReferenceType;
import com.hand.hap.core.web.view.UITag;
import com.hand.hap.core.web.view.ViewContext;
import com.hand.hap.core.web.view.XMap;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

/**
 * Created by gaoyang on 17/7/7.
 */
@UITag
public class Editor extends InputField {

    private static final String PROPERTITY_DESERIALIZATION = "deserialization";
    private static final String PROPERTITY_DOMAIN = "domain";
    private static final String PROPERTITY_ENCODED = "encoded";
    private static final String PROPERTITY_IMMUTABLES = "immutables";
    private static final String PROPERTITY_MESSAGES = "messages";
    private static final String PROPERTITY_PASTECLEANUP = "pasteCleanup";
    private static final String PROPERTITY_PDF = "pdf";
    private static final String PROPERTITY_RESIZABLE = "resizable";
    private static final String PROPERTITY_SERIALIZATION = "serialization";
    private static final String PROPERTITY_STYLESHEETS = "stylesheets";
    private static final String PROPERTITY_TOOLS = "tools";
    private static final String PROPERTITY_IMAGEBROWSER = "imageBrowser";
    private static final String PROPERTITY_FILEBROWSER = "fileBrowser";

    @Autowired
    private HlsComponentUtils hlsComponentUtils;

    public Map<String, ReferenceType> getDeserialization() {
        HashMap deserialization = new HashMap();
        deserialization.put("custom", new ReferenceType(this.getPrototype().getString(PROPERTITY_DESERIALIZATION)));
        return deserialization;
    }

    public void setDeserialization(ReferenceType deserialization) {
        this.setPropertity(PROPERTITY_DESERIALIZATION, deserialization);
    }

    public ReferenceType getDomain() {
        return ReferenceType.create(this.getPrototype().getString(PROPERTITY_DOMAIN));
    }

    public void setDomain(ReferenceType domain) {
        this.setPropertity(PROPERTITY_DOMAIN, domain);
    }

    public Boolean getEncoded() {
        return this.getPrototype().getBoolean(PROPERTITY_ENCODED);
    }

    public void setEncoded(boolean encoded) {
        this.setPropertity(PROPERTITY_ENCODED, encoded);
    }

    public Boolean getImmutables() {
        return this.getPrototype().getBoolean(PROPERTITY_IMMUTABLES);
    }

    public void setImmutables(Boolean immutables) {
        this.setPropertity(PROPERTITY_IMMUTABLES, immutables);
    }

    public Map<String,Object> getMessages() {
        String messages = this.getPrototype().getString(PROPERTITY_MESSAGES);
        return hlsComponentUtils.geStringMap(messages,null);
    }

    public void setMessages(String messages) {
        this.setPropertity(PROPERTITY_MESSAGES, messages);
    }

    public Map getPasteCleanup() {
        String pasteCleanup = this.getPrototype().getString(PROPERTITY_PASTECLEANUP);
        HashMap referenceMap = new HashMap();
        referenceMap.put("custom","referenceType");
        return hlsComponentUtils.geStringMap(pasteCleanup,referenceMap);
    }

    public void setPasteCleanup(String pasteCleanup) {
        this.setPropertity(PROPERTITY_PASTECLEANUP, pasteCleanup);
    }

    public Map<String,Object> getPdf() {
        String pdf=this.getPrototype().getString(PROPERTITY_PDF);
        return hlsComponentUtils.geStringMap(pdf,null);
    }

    public void setPdf(String pdf) {
        this.setPropertity(PROPERTITY_PDF, pdf);
    }

    public Map<String, Object> getResizable() {
        String resizable = this.getPrototype().getString(PROPERTITY_RESIZABLE);
        HashMap resizableMap = new HashMap();
        resizableMap.put("content","boolean");
        resizableMap.put("toolbar","boolean");
        return hlsComponentUtils.geStringMap(resizable,null);
    }

    public void setResizable(String resizable) {
        this.setPropertity(PROPERTITY_RESIZABLE, resizable);
    }

    public Map<String, Object> getSerialization() {
        String serialization = this.getPrototype().getString(PROPERTITY_SERIALIZATION);
        HashMap serializationMap = new HashMap();
        serializationMap.put("custom","referenceType");
        serializationMap.put("referenceType","boolean");
        serializationMap.put("scripts","boolean");
        serializationMap.put("semantic","boolean");
        return hlsComponentUtils.geStringMap(serialization,serializationMap);
    }

    public void setSerialization(String serialization) {
        this.setPropertity(PROPERTITY_SERIALIZATION, serialization);
    }

    public String[] getStylesheets() {
        String styleSheets = this.getPrototype().getString(PROPERTITY_STYLESHEETS);
        return hlsComponentUtils.geStringArray(styleSheets);
    }

    public void setStylesheets(String stylesheets) {
        this.setPropertity(PROPERTITY_STYLESHEETS, stylesheets);
    }

    public String getTools() {
        return this.getPrototype().getString(PROPERTITY_TOOLS);
    }

    public void setTools(String tools) {
        this.setPropertity(PROPERTITY_TOOLS, tools);
    }

    public String getImagebrowser() {
        return this.getPrototype().getString(PROPERTITY_IMAGEBROWSER);
    }

    public void setImagebrowser(String imagebrowser) {
        this.setPropertity(PROPERTITY_IMAGEBROWSER, imagebrowser);
    }

    public String getFilebrowser() {
        return this.getPrototype().getString(PROPERTITY_FILEBROWSER);
    }

    public void setFilebrowser(String filebrowser) {
        this.setPropertity(PROPERTITY_FILEBROWSER, filebrowser);
    }

    public static Editor createInstance() {
        XMap view = new XMap("h", "http://www.hand-china.com/hap", "editor");
        Editor editor = new Editor();
        editor.initPrototype(view);
        return editor;
    }

    public void init(XMap view, ViewContext context) throws Exception {
        super.init(view, context);
        context.addJsonConfig("deserialization", this.getDeserialization());
        context.addJsonConfig("encoded", this.getEncoded());
        context.addJsonConfig("immutables", this.getImmutables());
        context.addJsonConfig("messages", this.getMessages());
        context.addJsonConfig("pasteCleanup", this.getPasteCleanup());
        context.addJsonConfig("pdf", this.getPdf());
        context.addJsonConfig("resizable", this.getResizable());
        context.addJsonConfig("serialization", this.getSerialization());
        context.addJsonConfig("stylesheets", this.getStylesheets());
    }
}
